/* CC-222 Visao Computacional
   Autor: Carlos Henrique Q. Forster
   Permitido o uso sob a GPL. http://www.fsf.org/
*/

#include "vectors.h"

planar()
{
   int i;
   vec3 a,b,c, a1,b1,c1, a2,b2,c2, a3,b3,c3;
   vec3 origem,x,y;
   mat33 sc, T1, T2, T3;
   
   /*A figura eh um triangulo reto escaleno*/
   a=make3(3,2,1);
   b=make3(3,5,1);
   c=make3(5,2,1);
   
   /*A transformao T1*/
   T1=translation33(-3,-2);
   a1=apply33(T1,a);
   b1=apply33(T1,b);
   c1=apply33(T1,c);
   
   /*A transformao T2*/
   T2=rotation33(radians(45));
   a2=apply33(T2,a1);
   b2=apply33(T2,b1);
   c2=apply33(T2,c1);
   
   /*A transformao T3*/
   T3=translation33(3,2);
   a3=apply33(T3,a2);
   b3=apply33(T3,b2);
   c3=apply33(T3,c2);
   
   /*preparar os eixos coordenados*/
   origem=make3(0,0,1);
   x=make3(8,0,1);
   y=make3(0,8,1);
   
   /*ampliar tudo 10 vezes*/
   sc=scalemat33(10,10);
   a=apply33(sc,a);
   b=apply33(sc,b);
   c=apply33(sc,c);
   origem=apply33(sc,origem);
   x=apply33(sc,x);
   y=apply33(sc,y);
   a1=apply33(sc,a1);
   b1=apply33(sc,b1);
   c1=apply33(sc,c1);
   a2=apply33(sc,a2);
   b2=apply33(sc,b2);
   c2=apply33(sc,c2);
   a3=apply33(sc,a3);
   b3=apply33(sc,b3);
   c3=apply33(sc,c3);
   
   /*desenhar*/
   for(i=0;i<4;i++)
   {
      if(i==0) ps_start("planar0.eps");
      if(i==1) ps_start("planar1.eps");
      if(i==2) ps_start("planar2.eps");
      if(i==3) ps_start("planar3.eps");
      
      ps_arrow3("x",origem,x);
      ps_arrow3("y",origem,y);
      ps_dot3("O",origem);
   
      ps_color(0,0,255);
      ps_line3("",a,b);
      ps_line3("",a,c);
      ps_line3("",c,b);
   
      if(i==1)
      {
         ps_color(0,255,0);
         ps_line3("",a1,b1);
         ps_line3("",a1,c1);
         ps_line3("",c1,b1);
      }
    
      if(i==2)
      { 
         ps_color(255,100,0);
         ps_line3("",a2,b2);
         ps_line3("",a2,c2);
         ps_line3("",c2,b2);
      }
   
      if(i==3)
      {
         ps_color(0,255,100);
         ps_line3("",a3,b3);
         ps_line3("",a3,c3);
         ps_line3("",c3,b3);
      }
   
      ps_end();
   }
}


cube3d()
{
     int i;
     vec4 origin,minx,miny,minz,maxx,maxy,maxz;
     vec4 cube[8];
     mat44 transf,proj;
     
     /*para desenhar os eixos*/
     origin=make4(0,0,0,1);
     minx=make4(-3,0,0,1); maxx=make4(8,0,0,1);
     miny=make4(0,-3,0,1); maxy=make4(0,8,0,1);
     minz=make4(0,0,-3,1); maxz=make4(0,0,8,1);
     
     /*nossa figura eh um cubo unitario*/
     cube[0]=make4(0,0,0,1);
     cube[1]=make4(1,0,0,1);
     cube[2]=make4(1,1,0,1);
     cube[3]=make4(0,1,0,1);
     cube[4]=make4(0,1,1,1);
     cube[5]=make4(1,1,1,1);
     cube[6]=make4(1,0,1,1);
     cube[7]=make4(0,0,1,1);
     
     /*transformacao do cubo*/
     transf=identity44();
     transf=mul44(roty44(radians(45)),transf);
     transf=mul44(rotz44(radians(-45)),transf);
     transf=mul44(rotx44(radians(30)),transf);
     transf=mul44(rotz44(radians(45)),transf);
     transf=mul44(roty44(radians(-45)),transf);
     transf=mul44(scalemat44(2,2,2),transf);

     /*aplicar transformacao aos pontos do cubo*/
     for(i=0; i<8; i++)
     {
       cube[i]=cartesian4(apply44(transf,cube[i]));
     }
     
     /*calcular a projecao*/
     proj=identity44();
     proj=mul44(roty44(radians(-30)),proj);
     proj=mul44(rotx44(radians(30)),proj);
     proj=mul44(scalemat44(10,10,10),proj);
     proj=mul44(translation44(0,0,40),proj);
     proj=mul44(perspective44(90),proj);

     /*aplicar projecao aos pontos do cubo*/
     for(i=0; i<8; i++)
     {
       cube[i]=cartesian4(apply44(proj,cube[i]));
     }
     
     /*aplicar projecao aos pontos do eixo*/
     origin=cartesian4(apply44(proj,origin));
     minx=cartesian4(apply44(proj,minx));
     miny=cartesian4(apply44(proj,miny));
     minz=cartesian4(apply44(proj,minz));
     maxx=cartesian4(apply44(proj,maxx));
     maxy=cartesian4(apply44(proj,maxy));
     maxz=cartesian4(apply44(proj,maxz));
     
     /* desenhar */
     ps_start("mydrawing.ps");
     ps_dot("O",origin.x,origin.y);
     ps_arrow("x",minx.x,minx.y,maxx.x,maxx.y);
     ps_arrow("y",miny.x,miny.y,maxy.x,maxy.y);
     ps_arrow("z",minz.x,minz.y,maxz.x,maxz.y);
     
     ps_line("",-240,-140,240,-140);
     ps_line("",-240,140,240,140);
     ps_line("",-240,-140,-240,140);
     ps_line("",240,-140,240,140);

     ps_color(255,0,0);
     ps_linewidth(1);
     for(i=0; i<8; i++) ps_dot4("",cube[i]);
     ps_line4("",cube[0],cube[1]);
     ps_line4("",cube[1],cube[2]);
     ps_line4("",cube[2],cube[3]);
     ps_line4("",cube[3],cube[0]);
     ps_line4("",cube[0],cube[7]);
     ps_line4("",cube[1],cube[6]);
     ps_line4("",cube[2],cube[5]);
     ps_line4("",cube[3],cube[4]);
     ps_line4("",cube[6],cube[7]);
     ps_line4("",cube[5],cube[6]);
     ps_line4("",cube[4],cube[5]);
     ps_line4("",cube[7],cube[4]);
     
     ps_end();
}      


vanishing()
{
     int i;
     vec4 origin,minx,miny,minz,maxx,maxy,maxz;
     vec4 cube[8], vanish[3];
     mat44 transf,proj;
     
     /*para desenhar os eixos*/
     origin=make4(0,0,0,1);
     minx=make4(-3,0,0,1); maxx=make4(8,0,0,1);
     miny=make4(0,-3,0,1); maxy=make4(0,8,0,1);
     minz=make4(0,0,-3,1); maxz=make4(0,0,8,1);
     
     /*nossa figura eh um cubo unitario*/
     cube[0]=make4(0,0,0,1);
     cube[1]=make4(1,0,0,1);
     cube[2]=make4(1,1,0,1);
     cube[3]=make4(0,1,0,1);
     cube[4]=make4(0,1,1,1);
     cube[5]=make4(1,1,1,1);
     cube[6]=make4(1,0,1,1);
     cube[7]=make4(0,0,1,1);
     
     /*pontos de fuga do cubo*/
     vanish[0]=make4(1,0,0,0);
     vanish[1]=make4(0,1,0,0);
     vanish[2]=make4(0,0,1,0);
     
     /*escala do cubo*/
     transf=identity44();
     transf=mul44(scalemat44(4,2,3),transf);
     transf=mul44(translation44(0.1,-3,0.2),transf);

     /*aplicar transformacao aos pontos do cubo*/
     for(i=0; i<8; i++)
     {
       cube[i]=(apply44(transf,cube[i]));
     }
     for(i=0; i<3; i++)
     {
       vanish[i]=(apply44(transf,vanish[i]));
     }
     
     /*construir projecao baseada nos pontos de fuga*/
     proj=transp44(make44(-50,30,0,1, 2,50,0,0.1, 100,30,0,1, 0,0,0,2 ));

     /*aplicar projecao aos pontos do cubo*/
     for(i=0; i<8; i++)
     {
       cube[i]=(apply44(proj,cube[i]));
     }
     for(i=0; i<3; i++)
     {
       vanish[i]=(apply44(proj,vanish[i]));
     }
     
     /*aplicar projecao aos pontos do eixo*/
     origin=cartesian4(apply44(proj,origin));
     minx=cartesian4(apply44(proj,minx));
     miny=cartesian4(apply44(proj,miny));
     minz=cartesian4(apply44(proj,minz));
     maxx=cartesian4(apply44(proj,maxx));
     maxy=cartesian4(apply44(proj,maxy));
     maxz=cartesian4(apply44(proj,maxz));
     
     /* desenhar */
     ps_start("vanishing.ps");
     ps_dot("O",origin.x,origin.y);
     ps_arrow("x",minx.x,minx.y,maxx.x,maxx.y);
     ps_arrow("y",miny.x,miny.y,maxy.x,maxy.y);
     ps_arrow("z",minz.x,minz.y,maxz.x,maxz.y);
     
     ps_line("",-240,-140,240,-140);
     ps_line("",-240,140,240,140);
     ps_line("",-240,-140,-240,140);
     ps_line("",240,-140,240,140);

     ps_color(255,0,0);
     ps_linewidth(1);
     for(i=0; i<8; i++) ps_dot4("",cube[i]);
     ps_line4("",cube[0],cube[1]);
     ps_line4("",cube[1],cube[2]);
     ps_line4("",cube[2],cube[3]);
     ps_line4("",cube[3],cube[0]);
     ps_line4("",cube[0],cube[7]);
     ps_line4("",cube[1],cube[6]);
     ps_line4("",cube[2],cube[5]);
     ps_line4("",cube[3],cube[4]);
     ps_line4("",cube[6],cube[7]);
     ps_line4("",cube[5],cube[6]);
     ps_line4("",cube[4],cube[5]);
     ps_line4("",cube[7],cube[4]);
     
     ps_box4("fx",vanish[0]);
     ps_box4("fy",vanish[1]);
     ps_box4("fz",vanish[2]);
     
     ps_end();
}      



main()
{
      planar();
      cube3d();
      vanishing();
}
