/* CC-222 Visao Computacional
   Autor: Carlos Henrique Q. Forster
   Permitido o uso sob a GPL. http://www.fsf.org/
*/

#include "vectors.h"
#include <stdio.h>
#include <math.h>

vec3 make3(float x, float y, float z)
{
     vec3 r;
     r.x=x;
     r.y=y;
     r.z=z;
     return r;
}

vec4 make4(float x, float y, float z,float w)
{
     vec4 r;
     r.x=x;
     r.y=y;
     r.z=z;
     r.w=w;
     return r;
}

mat33 make33(float a11, float a12, float a13,
             float a21, float a22, float a23,
             float a31, float a32, float a33)
{
     mat33 m;
     m.x=make3(a11,a12,a13);
     m.y=make3(a21,a22,a23);
     m.z=make3(a31,a32,a33);
     return m;
}

mat44 make44(float a11, float a12, float a13, float a14,
             float a21, float a22, float a23, float a24,
             float a31, float a32, float a33, float a34,
             float a41, float a42, float a43, float a44)
{
     mat44 m;
     m.x=make4(a11,a12,a13,a14);
     m.y=make4(a21,a22,a23,a24);
     m.z=make4(a31,a32,a33,a34);
     m.w=make4(a41,a42,a43,a44);
     return m;
}


mat33 makev33(vec3 x, vec3 y, vec3 z)
{
     mat33 r;
     r.x=x;
     r.y=y;
     r.z=z;
     return r;
}

mat44 makev44(vec4 x, vec4 y, vec4 z, vec4 w)
{
     mat44 r;
     r.x=x;
     r.y=y;
     r.z=z;
     r.w=w;
     return r;
}

void print3(vec3 a)
{
     printf("(%.2f,%.2f,%.2f)",a.x,a.y,a.z);
}

void print4(vec4 a)
{
     printf("(%.2f,%.2f,%.2f,%.2f)",a.x,a.y,a.z,a.w);
}

void print33(mat33 a)
{
     printf("("); print3(a.x);
     printf(","); print3(a.y);
     printf(","); print3(a.z);
     printf(")");
}

void print44(mat44 a)
{
     printf("("); print4(a.x);
     printf(","); print4(a.y);
     printf(","); print4(a.z);
     printf(","); print4(a.w);
     printf(")");
}

vec3 add3(vec3 a, vec3 b) {return make3(a.x+b.x,a.y+b.y,a.z+b.z);}
vec3 neg3(vec3 a) {return make3(-a.x,-a.y,-a.z); }
vec3 sub3(vec3 a, vec3 b) {return make3(a.x-b.x,a.y-b.y,a.z-b.z); }
vec3 scale3(float s, vec3 a) {return make3(s*a.x,s*a.y,s*a.z); }
float dot3(vec3 a, vec3 b) {return a.x*b.x+a.y*b.y+a.z*b.z; }
vec3 cross3(vec3 a, vec3 b)
{
     return make3( a.y*b.z-b.y*a.z, a.z*b.x-b.z*a.x, a.x*b.y-a.y*b.x );
}
float length3(vec3 a) {return sqrt(dot3(a,a));}
vec3 dir3(vec3 a) {return scale3(1/length3(a),a);}
vec3 cartesian3(vec3 a) {return make3(a.x/a.z, a.y/a.z, 1);} 

vec4 add4(vec4 a, vec4 b) {return make4(a.x+b.x,a.y+b.y,a.z+b.z,a.w+b.w);}
vec4 neg4(vec4 a) {return make4(-a.x,-a.y,-a.z,-a.w); }
vec4 sub4(vec4 a, vec4 b) {return make4(a.x-b.x,a.y-b.y,a.z-b.z,a.w-b.w); }
vec4 scale4(float s, vec4 a) {return make4(s*a.x,s*a.y,s*a.z,s*a.w); }
float dot4(vec4 a, vec4 b) {return a.x*b.x+a.y*b.y+a.z*b.z+a.w*b.w; }
vec4 cross4(vec4 a, vec4 b, vec4 c)
{
     return neg4(
       make4( 
           -a.y*b.z*c.w+a.y*b.w*c.z+b.y*a.z*c.w-b.y*a.w*c.z-c.y*a.z*b.w+c.y*a.w*b.z,
           a.x*b.z*c.w-a.x*b.w*c.z-b.x*a.z*c.w+b.x*a.w*c.z+c.x*a.z*b.w-c.x*a.w*b.z,
           -a.x*b.y*c.w+a.x*c.y*b.w+b.x*a.y*c.w-b.x*c.y*a.w-c.x*a.y*b.w+c.x*b.y*a.w,
           a.x*b.y*c.z-a.x*c.y*b.z-b.x*a.y*c.z+b.x*c.y*a.z+c.x*a.y*b.z-c.x*b.y*a.z));
}
float length4(vec4 a) {return sqrt(dot4(a,a));}
vec4 dir4(vec4 a) {return scale4(1/length4(a),a);}
vec4 cartesian4(vec4 a) {return make4(a.x/a.w, a.y/a.w, a.z/a.w, 1);} 
vec4 augment4(vec3 a) {return make4(a.x,a.y,a.z,1);}

mat33 add33(mat33 a, mat33 b) {return makev33(add3(a.x,b.x),add3(a.y,b.y),add3(a.z,b.z));}
mat33 neg33(mat33 a) {return makev33(neg3(a.x),neg3(a.y),neg3(a.z));}
mat33 sub33(mat33 a, mat33 b) {return makev33(sub3(a.x,b.x),sub3(a.y,b.y),sub3(a.z,b.z));}
mat33 scale33(float s, mat33 a) {return makev33(scale3(s,a.x),scale3(s,a.y),scale3(s,a.z));}
mat33 transp33(mat33 a)
{
      return make33(a.x.x,a.y.x,a.z.x, a.x.y,a.y.y,a.z.y, a.x.z,a.y.z,a.z.z);
}

mat44 add44(mat44 a, mat44 b) {return makev44(add4(a.x,b.x),add4(a.y,b.y),add4(a.z,b.z),add4(a.w,b.w));}
mat44 neg44(mat44 a) {return makev44(neg4(a.x),neg4(a.y),neg4(a.z),neg4(a.w));}
mat44 sub44(mat44 a, mat44 b) {return makev44(sub4(a.x,b.x),sub4(a.y,b.y),sub4(a.z,b.z),sub4(a.w,b.w));}
mat44 scale44(float s, mat44 a) {return makev44(scale4(s,a.x),scale4(s,a.y),scale4(s,a.z),scale4(s,a.w));}
mat44 transp44(mat44 a)
{
      return make44(a.x.x,a.y.x,a.z.x,a.w.x,
                    a.x.y,a.y.y,a.z.y,a.w.y,
                    a.x.z,a.y.z,a.z.z,a.w.z,
                    a.x.w,a.y.w,a.z.w,a.w.w);
}


mat33 mul33(mat33 a, mat33 b)
{
      b=transp33(b);
      return make33(dot3(a.x,b.x),dot3(a.x,b.y),dot3(a.x,b.z),
                    dot3(a.y,b.x),dot3(a.y,b.y),dot3(a.y,b.z),
                    dot3(a.z,b.x),dot3(a.z,b.y),dot3(a.z,b.z));
}

float det33(mat33 a)
{
      return dot3(a.x,cross3(a.y,a.z));
}

mat33 cofactor33(mat33 a)
{
      return makev33(cross3(a.y,a.z),cross3(a.z,a.x),cross3(a.x,a.y));
}

mat33 adjoint33(mat33 a)
{
      return transp33(cofactor33(a));
}

mat33 inv33(mat33 a)
{
      return scale33(1.0f/det33(a),adjoint33(a));
}

vec3 apply33(mat33 m, vec3 a)
{
      return make3(dot3(m.x,a),dot3(m.y,a),dot3(m.z,a));
}

mat44 mul44(mat44 a, mat44 b)
{
      b=transp44(b);
      return make44(dot4(a.x,b.x),dot4(a.x,b.y),dot4(a.x,b.z),dot4(a.x,b.w),
                    dot4(a.y,b.x),dot4(a.y,b.y),dot4(a.y,b.z),dot4(a.y,b.w),
                    dot4(a.z,b.x),dot4(a.z,b.y),dot4(a.z,b.z),dot4(a.z,b.w),
                    dot4(a.w,b.x),dot4(a.w,b.y),dot4(a.w,b.z),dot4(a.w,b.w));
}

float det44(mat44 a)
{
      return dot4(a.x,cross4(a.y,a.z,a.w));
}

mat44 cofactor44(mat44 a)
{
      return makev44(cross4(a.y,a.z,a.w),cross4(a.w,a.z,a.x),
                     cross4(a.w,a.x,a.y),cross4(a.y,a.x,a.z));
}

mat44 adjoint44(mat44 a)
{
      return transp44(cofactor44(a));
}

mat44 inv44(mat44 a)
{
      return scale44(1.0f/det44(a),adjoint44(a));
}

vec4 apply44(mat44 m, vec4 a)
{
      return make4(dot4(m.x,a),dot4(m.y,a),dot4(m.z,a),dot4(m.w,a));
}


mat33 scalemat33(float sx, float sy)
{
      return make33(sx,0,0, 0,sy,0, 0,0,1);
}

mat33 translation33(float x, float y)
{
      return make33(1,0,x, 0,1,y, 0,0,1);
}

mat33 rotation33(float t)
{
      return make33(cos(t),-sin(t),0, sin(t),cos(t),0, 0,0,1);
}

mat33 mirrorx33(float t)
{
      return make33(1,0,0, 0,-1,0, 0,0,1);
}

mat33 identity33()
{
      return make33(1,0,0, 0,1,0, 0,0,1);
}

mat44 augment44(mat33 m)
{
      return make44(m.x.x,m.x.y,m.x.z,0,
                    m.y.x,m.y.y,m.y.z,0,
                    m.z.x,m.z.y,m.z.z,0,
                    0,0,0,1);
}

mat44 scalemat44(float sx, float sy, float sz)
{
      return make44(sx,0,0,0, 0,sy,0,0, 0,0,sz,0, 0,0,0,1);
}

mat44 translation44(float x, float y, float z)
{
      return make44(1,0,0,x, 0,1,0,y, 0,0,1,z, 0,0,0,1);
}

mat44 translation_to_origin44(vec4 p)
{
      return make44(p.w,0,0,-p.x, 0,p.w,0,-p.y, 0,0,p.w,-p.z, 0,0,0,p.w);
}

mat44 rotx44(float t)
{
      float c=cos(t);
      float s=sin(t);
      return make44(1,0,0,0, 0,c,-s,0, 0,s,c,0, 0,0,0,1);
}

mat44 roty44(float t)
{
      float c=cos(t);
      float s=sin(t);
      return make44(c,0,s,0, 0,1,0,0, -s,0,c,0, 0,0,0,1);
}

mat44 rotz44(float t)
{
      float c=cos(t);
      float s=sin(t);
      return make44(c,-s,0,0, s,c,0,0, 0,0,1,0, 0,0,0,1);
}

mat44 rotframe44(vec3 vpn, vec3 vup)
{
      vec3 vx,vy;
      vpn=dir3(vpn);
      vx=dir3(cross3(vup,vpn));
      vy=cross3(vpn,vx);
      return make44(vx.x,vy.x,vpn.x,0,
                    vx.y,vy.y,vpn.y,0,
                    vx.z,vy.z,vpn.z,0,
                    0,0,0,1);
}

mat44 mirrorxy44()
{
      return make44(1,0,0,0, 0,1,0,0, 0,0,-1,0, 0,0,0,1);
}

mat44 identity44()
{
      return make44(1,0,0,0, 0,1,0,0, 0,0,1,0, 0,0,0,1);
}

mat44 perspective44(float f)
{
      return make44(f,0,0,0, 0,f,0,0, 0,0,f,0, 0,0,1,0);
}

mat44 perspframe44(vec4 projcenter, vec4 lookat, float focal)
{
      mat44 transl, rot, proj;
      vec3 vpn,vup;
      
      transl=translation_to_origin44(projcenter);
      lookat=apply44(transl,lookat);
      vpn=dir3(make3(lookat.x,lookat.y,lookat.z));
      vup=make3(0,1,0);
      rot=rotframe44(vpn,vup);
      proj=perspective44(focal);
      return mul44(proj,mul44(rot,transl));
}

FILE *psfile, *pstemplate;

void ps_start(char* filename)
{
     psfile=fopen(filename,"w");
     if(!psfile)
     {
        printf("Cant create PS file");
        exit(2);
     }
     pstemplate=fopen("vectors.pre","r");
     if(!psfile)
     {
        printf("Cant find vectors.pre file");
        exit(3);
     }
     int c;
     while (c=getc(pstemplate),c!=EOF) putc(c,psfile);
     fclose(pstemplate);
}

void ps_end()
{
    fprintf(psfile,"\nshowpage\n%%Trailer\n");
    fclose(psfile);
}

void ps_dot(char* label, float x, float y)
{
     fprintf(psfile,"(%s) %f %f dot\n",label,x,y);
}

void ps_box(char* label, float x, float y)
{
     fprintf(psfile,"(%s) %f %f box\n",label,x,y);
}

void ps_linewidth(float wid)
{
     fprintf(psfile,"%f setlinewidth\n",wid);
}

void ps_color(int r, int g, int b)
{
     fprintf(psfile,"%d %d %d setrgbcolor\n",r,g,b);
}

void ps_font(char* name, int size)
{
     fprintf(psfile,"/%s findfont %d scalefont setfont\n",name,size);
}

void ps_scale(float wid)
{
     fprintf(psfile,"%f %f scale\n",wid,wid);
}

void ps_arrow(char* label, float x0, float y0, float x1, float y1)
{
     fprintf(psfile,"(%s) %f %f %f %f arrow\n",label,x0,y0,x1,y1);
}

void ps_line(char* label, float x0, float y0, float x1, float y1)
{
     fprintf(psfile,"(%s) %f %f %f %f line\n",label,x0,y0,x1,y1);
}

void ps_dot3(char *label, vec3 p)
{
    p=cartesian3(p);
    ps_dot(label,p.x,p.y);
}

void ps_box3(char *label, vec3 p)
{
    p=cartesian3(p);
    ps_box(label,p.x,p.y);
}

void ps_arrow3(char *label, vec3 p1, vec3 p2)
{
    p1=cartesian3(p1); p2=cartesian3(p2);
    ps_arrow(label, p1.x,p1.y,p2.x,p2.y);
}

void ps_line3(char *label, vec3 p1, vec3 p2)
{
    p1=cartesian3(p1); p2=cartesian3(p2);
    ps_line(label, p1.x,p1.y,p2.x,p2.y);
}


void ps_dot4(char *label, vec4 p)
{
    p=cartesian4(p);
    ps_dot(label,p.x,p.y);
}

void ps_box4(char *label, vec4 p)
{
    p=cartesian4(p);
    ps_box(label,p.x,p.y);
}

void ps_arrow4(char *label, vec4 p1, vec4 p2)
{
    p1=cartesian4(p1); p2=cartesian4(p2);
    ps_arrow(label, p1.x,p1.y,p2.x,p2.y);
}

void ps_line4(char *label, vec4 p1, vec4 p2)
{
    p1=cartesian4(p1); p2=cartesian4(p2);
    ps_line(label, p1.x,p1.y,p2.x,p2.y);
}

float degrees(float rad)
{
      return 180.0f*rad/M_PI;
}

float radians(float deg)
{
      return deg*M_PI/180.0f;
}


void test()
{
      mat33 a,b;
      mat44 c,d;
      vec4 u;
      vec3 v;
      a=make33(1,2,3,3,3,1,1,2,2);
      b=make33(0,1,2,3,4,6,7,8,9);
      v=make3(4,0,2);
      c=make44(2,4,6,2, 2,4,2,4, 4,1,2,2, 6,4,2,1);
      print33(a); printf("=a\n");
      print33(b); printf("=b\n");
      print33(transp33(a)); printf("=transp(a)\n");
      print33(mul33(a,b)); printf("=a*b\n");
      print3(v); printf("=v\n");
      print3(apply33(a,v)); printf("=a*v\n");
      printf("det(a): %g\n",det33(a));
      print33(inv33(a)); printf("=inv(a)\n");
      print33(mul33(a,inv33(a))); printf("=a*inv(a)\n");
      print44(c); printf("=c\n");
      print44(transp44(c)); printf("=transp(c)\n");
      print44(adjoint44(c)); printf("=adjoint(c)\n");
      print44(mul44(c,inv44(c))); printf("=c*inv(c)\n");
      printf("det(c): %g\n",det44(c));
      
      getchar();
}

