/* CC-222 Visao Computacional
   Autor: Carlos Henrique Q. Forster
   Permitido o uso sob a GPL. http://www.fsf.org/
*/
#ifndef _VECTORS_H
#define _VECTORS_H

#include <stdio.h>
#include <math.h>

typedef struct{ float x,y,z; } vec3;
typedef struct{ vec3 x,y,z; } mat33;
typedef struct{ float x,y,z,w; } vec4;
typedef struct{ vec4 x,y,z,w; } mat44;

vec3 make3(float x, float y, float z);
vec4 make4(float x, float y, float z,float w);
mat33 make33(float a11, float a12, float a13,
             float a21, float a22, float a23,
             float a31, float a32, float a33);
mat44 make44(float a11, float a12, float a13, float a14,
             float a21, float a22, float a23, float a24,
             float a31, float a32, float a33, float a34,
             float a41, float a42, float a43, float a44);
mat33 makev33(vec3 x, vec3 y, vec3 z);
mat44 makev44(vec4 x, vec4 y, vec4 z, vec4 w);

void print3(vec3 a);
void print4(vec4 a);
void print33(mat33 a);
void print44(mat44 a);

vec3 add3(vec3 a, vec3 b);
vec3 neg3(vec3 a);
vec3 sub3(vec3 a, vec3 b);
vec3 scale3(float s, vec3 a);
float dot3(vec3 a, vec3 b);
vec3 cross3(vec3 a, vec3 b);
float length3(vec3 a);
vec3 dir3(vec3 a);
vec3 cartesian3(vec3 a);

vec4 add4(vec4 a, vec4 b);
vec4 neg4(vec4 a);
vec4 sub4(vec4 a, vec4 b);
vec4 scale4(float s, vec4 a);
float dot4(vec4 a, vec4 b);
vec4 cross4(vec4 a, vec4 b, vec4 c);
float length4(vec4 a);
vec4 dir4(vec4 a);
vec4 cartesian4(vec4 a);
vec4 augment4(vec3 a);

mat33 add33(mat33 a, mat33 b);
mat33 neg33(mat33 a);
mat33 sub33(mat33 a, mat33 b);
mat33 scale33(float s, mat33 a);
mat33 transp33(mat33 a);

mat44 add44(mat44 a, mat44 b);
mat44 neg44(mat44 a);
mat44 sub44(mat44 a, mat44 b);
mat44 scale44(float s, mat44 a);
mat44 transp44(mat44 a);

mat33 mul33(mat33 a, mat33 b);
float det33(mat33 a);
mat33 cofactor33(mat33 a);
mat33 adjoint33(mat33 a);
mat33 inv33(mat33 a);

vec3 apply33(mat33 m, vec3 a);

mat44 mul44(mat44 a, mat44 b);
float det44(mat44 a);
mat44 cofactor44(mat44 a);
mat44 adjoint44(mat44 a);
mat44 inv44(mat44 a);

vec4 apply44(mat44 m, vec4 a);

mat33 scalemat33(float sx, float sy);
mat33 translation33(float x, float y);
mat33 rotation33(float t);
mat33 mirrorx33(float t);
mat33 identity33();

mat44 augment44(mat33 m);
mat44 scalemat44(float sx, float sy, float sz);
mat44 translation44(float x, float y, float z);
mat44 translation_to_origin44(vec4 p);
mat44 rotx44(float t);
mat44 roty44(float t);
mat44 rotz44(float t);
mat44 rotframe44(vec3 vpn, vec3 vup);
mat44 mirrorxy44();
mat44 identity44();
mat44 perspective44(float f);
mat44 perspframe44(vec4 projcenter, vec4 lookat, float focal);

extern FILE *psfile, *pstemplate;

void ps_start(char* filename);
void ps_end();
void ps_dot(char* label, float x, float y);
void ps_box(char* label, float x, float y);
void ps_linewidth(float wid);
void ps_color(int r, int g, int b);
void ps_font(char* name, int size);
void ps_scale(float wid);
void ps_arrow(char* label, float x0, float y0, float x1, float y1);
void ps_line(char* label, float x0, float y0, float x1, float y1);
void ps_dot3(char *label, vec3 p);
void ps_box3(char *label, vec3 p);
void ps_arrow3(char *label, vec3 p1, vec3 p2);
void ps_line3(char *label, vec3 p1, vec3 p2);
void ps_dot4(char *label, vec4 p);
void ps_box4(char *label, vec4 p);
void ps_arrow4(char *label, vec4 p1, vec4 p2);
void ps_line4(char *label, vec4 p1, vec4 p2);

float degrees(float rad);
float radians(float deg);

void test();

#endif
